from __future__ import nested_scopes

import compiler
import tokenutils
import tokenize
import token
import brmtransformer

def addTokens(ast, src):
    compiler.walk(ast, AddTokens(src))


class AddTokens:

    def __init__(self, src):
        tlist = tokenutils.getTokenList(src)
        self.tc = tokenutils.TokenConverter(tlist)

    def visitSubNodesWithTokens(self, nodes, toks, dividerTokenName = ":"):
        oldtokenconverter = self.tc
        self.tc = tokenutils.TokenConverter(toks)

        if dividerTokenName == ":":
            tbefore = self.tc.getTokensStartBlock()
        else:
            tbefore = self.tc.getTokensTo(dividerTokenName)
        for node in nodes:
            self.visit(node)
        tafter = self.tc.getRestOfTokens()
        if len(tafter) != 0and tafter[-1].type == tokenize.ENDMARKER:
            tafter = tafter[:-1]
        self.tc = oldtokenconverter
        return tbefore, tafter


    def visitCompoundNode(self, node):
        toks = self.tc.getTokens(node.parsetree)
        toks+=self.tc.getWhitespaceLines()
        tbefore, tafter = self.visitSubNodesWithTokens([node.code], toks)
        node.addTokens(tbefore, tafter)


    def visitModule(self, node):
        tbefore = self.tc.getTokensUptoLastNewlineBeforeCode()
        self.visit(node.node)
        node.addTokens(tbefore, self.tc.getRestOfTokens())

    def visitClass(self, node):
        toks = self.tc.getTokens(node.parsetree)
        toks+=self.tc.getWhitespaceLines()

        if node.bases != []:
            upToBaseSpec, tafter = self.visitSubNodesWithTokens(node.bases, toks, "(")
            afterBaseSpec, tafter = self.visitSubNodesWithTokens([node.code], tafter)
            node.addTokens(upToBaseSpec, afterBaseSpec, tafter)
        else:
            tbefore, tafter = self.visitSubNodesWithTokens([node.code], toks)
            node.addTokens(tbefore, [], tafter)

    def visitFunction(self, node):
        self.visitCompoundNode(node)

    def visitBasicNode(self, node):
        toks = self.tc.getTokens(node.parsetree)
        toks+=self.tc.getCommentAndNewline()

        # get blanklines
        while self.tc.peekNextToken()and \
              self.tc.peekNextToken().type == tokenize.NL:
            toks.append(self.tc.popToken())

        node.addTokens(toks)

    def visitPass(self, node):
        self.visitBasicNode(node)


    def visitGetattr(self, node):
        preceedingtokens = self.tc.getTokensPreceedingNodes(node.parsetree)
        toks = self.tc.getTokens(node.parsetree)
        toks+=self.tc.getCommentAndNewline()
        # get blanklines
        while self.tc.peekNextToken()and \
              self.tc.peekNextToken().type == tokenize.NL:
            toks.append(self.tc.popToken())

        # call children (to handle nested getattrs)
        # N.B. this is a hack to ensure that stuff still works if children
        # don't take their tokens off the list
        # this needs replacing with a global strategy - See TODO
        oldtokenconverter = self.tc
        self.tc = tokenutils.TokenConverter(toks)
        self.visit(node.expr)
        node.addTokens(preceedingtokens, self.tc.getRestOfTokens())
        self.tc = oldtokenconverter

    def visitName(self, node):
        self.visitBasicNode(node)

    def visitImport(self, node):
        self.visitBasicNode(node)

    def visitFrom(self, node):
        self.visitBasicNode(node)
