#!/usr/bin/env python 
import setpath
import unittest
import os
from bike.parsing.extended_ast import Top, Source

from test_load import tmpfile, writeTmpTestFile

from bike import testdata
from tokenutils import getTokenList
from bike.mock import Mock
from brmtransformer import parse
from output import*
from addtokens import addTokens
from bike.testutils import*



class TestTokensToString(unittest.TestCase):
    def testTokensToString_withSource(self):
        src = testdata.source_with_all_ast_nodes[1:]
        ast = parse(src)
        addTokens(ast, src)
        newsrc = astToString(ast)
        assert newsrc == src

    def testTokensToString_withClass(self):
        src = testdata_class[1:]
        ast = parse(src)
        addTokens(ast, src)
        assert(src == astToString(ast))

    def testTokensToString_withClassAndMethod(self):
        src = testdata.class_and_method[1:]
        ast = parse(src)
        addTokens(ast, src)
        out = astToString(ast)
        assert(src == out)

    # I like my code to be formatted  a = b, not a=b
    def testTokensToString_addsASpaceWhenHandlingAssignments(self):
        src = "a = b"
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src

    # a.b, not a. b
    def testTokensToString_putsNoSpacesInGetAttrDot(self):
        src = "a.b"
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src

    # a == b, not a==b
    def testTokensToString_addsSpacesWhenHandlingConditionals(self):
        for src in "a == b", "a != b", "a < b", "a <= b", "a > b", "a >= b", "a in b":
            ast = parse(src)
            addTokens(ast, src)
            assert astToString(ast) == src, "adds space when handling "+src

    def testTokensToString_addsSpaceAfterColonWhenDoingInlineBlocks(self):
        src = "def foo(): pass"
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src

    def testTokensToString_addsTwoSpacesWhenDoingInlineComments(self):
        src = "pass  # comment"
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src

    def testTokensToString_addsASpaceAfterEachComma(self):
        src = testdata.StreamToStderr[1:]
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src

    def testTokensToString_handlesLineBreaks(self):
        src = testdata.linebreak[1:]
        ast = parse(src)
        addTokens(ast, src)
        assert astToString(ast) == src
    
    def testTokensToString_handlesLineBreaksIfTokenHasBeenMoved(self):
        src = testdata.linebreak[1:]
        ast = parse(src)
        addTokens(ast, src)
        tokens = ast.getTokens()
        from bike.parsing.tokenutils import TokenList
        # modify the token
        tokens[12] = TokenList.token(tokenize.NAME,'babble',(3,1),(0,0),"babble")
        assert tokensToString(tokens) == testdata.linebreak_outputIfTokensMoved[1:]

    def testTokensToString_returnsEmptyStringIfGivenEmptyList(self):
        assert tokensToString([]) == ""


testdata_class = """
class foo:
    pass
"""



class TestCommentIncludesNewline(unittest.TestCase):

    def testCommentIncludesNewline_returnsTrueIfCommentEndsInNewline(self):
        src = testdata_comment_ending_in_newline[1:]
        tokens = getTokenList(src)
        assert TokensToString().commentIncludesNewline(tokens[0])

    def testCommentIncludesNewline_returnsFalseIfDoesntContainNewline(self):
        src = testdata_comment_not_ending_in_newline[1:]
        tokens = getTokenList(src)
        assert TokensToString().commentIncludesNewline(tokens[1]) == 0



testdata_comment_ending_in_newline = """
# comment
"""

testdata_comment_not_ending_in_newline = """
pass # comment
"""


class TestSave(unittest.TestCase):
    def testSave_doesntWriteFilesIfSourceNotModified(self):
        writeTmpTestFile("notpass")
        s = Source(tmpfile, "foobahfile", "pass")
        xast = Top([], [s], "")
        save(xast)  # setModified() not called on source!
        open(tmpfile).read() == "notpass"  # file hasnt changed
        os.remove(tmpfile)


    def testSave_writesFilesIfSourceModified(self):
        writeTmpTestFile("notpass")
        s = Source(tmpfile, "foobahfile", "pass")
        xast = Top([], [s], "")
        s.tokenize()
        s.setModified()
        filesWritten = save(xast)
        assert open(tmpfile).read() == "pass"        
        os.remove(tmpfile)
        assert filesWritten == [os.path.abspath(tmpfile)]




if __name__ == "__main__":
    unittest.main()
