import os
import os.path
from mock import Mock
from bike.parsing.extended_ast import Package, Source, PackageAlreadyExistsException, Top
from parsing.brmtransformer import parse
from parsing.addtokens import addTokens
from parsing.addtypeinfo import addtypeinfo
from parsing.utils import fqn_rcar,fqn_rcdr

tmpfile = "bicyclerepairman_tmp_testfile.py"
tmpmodule = "bicyclerepairman_tmp_testfile"


def writeFile(filename,src):
    f = open(filename,"w+")
    f.write(src)
    f.close()

def readFile(filename):
    f = open(filename)
    src = f.read()
    f.close()
    return src

def writeTmpTestFile(src):
    writeFile(tmpfile,src)

def readTmpTestFile():
    return readFile(tmpfile)

def deleteTmpTestFile():
    os.remove(tmpfile)


def createPackageStructure(src1,src2):
    try: os.makedirs(os.path.join("a","b"))
    except: pass
    writeFile(os.path.join("a","foo.py"),src1)
    writeFile(os.path.join("a","b","bah.py"),src2)

def removePackageStructure():
    os.remove(os.path.join("a","foo.py"))
    os.remove(os.path.join("a","b","bah.py"))
    os.removedirs(os.path.join("a","b"))


def createSourceNodeAt(top,src,fqn):
    node = top
    modname = fqn_rcar(fqn)
    packagefqn = fqn_rcdr(fqn)
    if packagefqn != "":
        for packagename in packagefqn.split("."):
            try:
                node.addPackage(Package(packagename,packagename))
            except PackageAlreadyExistsException:
                pass
            node = node.getPackage(packagename)
    node.addSource(Source(modname,modname,src))
    return top


# uses compiler visitor interface along with some dynamic magic
# to get a node out of the ast tree obtained from the src
# N.B. index refers to the number of nodes to count before returning.
# e.g. for second function, nodename=Function, index=1
def getASTNodeFromSrc(src,nodename,index=0):
    from bike.parsing.brmtransformer import parse
    ast = Top()
    ast.addSource(Source("foo.py","foo",src))
    addTokens(ast,src)
    addtypeinfo(ast)
    return getASTNode(ast,nodename,index)

def getASTNode(ast,nodename,index=0):
    import compiler
    class FindNode:
        def __init__(self,nodename):
            self.nodename = nodename
            self.index = 0
        def __getattr__(self,name):
            if(name == "visit"+self.nodename):
                return self.visitNode
            else:
                assert name != "node"
                raise AttributeError,name
        def visitNode(self,node,scope=0):
            if self.index == index:
                self.node = node
            self.index +=1
            for child in node.getChildNodes():
                self.visit(child)

    f = FindNode(nodename)
    compiler.walk(ast,f)
    return f.node



def getMatchingASTNode(ast,nodename,**predicates):
    import compiler
    class FindNode:
        def __init__(self,nodename):
            self.nodename = nodename
            self.node = None
        def __getattr__(self,name):
            if(name == "visit"+self.nodename):
                return self.visitNode
            else:
                assert name != "node"
                raise AttributeError,name

        def _nodeMatchesPredicate(self,node,name,value):
            return hasattr(node,name) and getattr(node,name) == value

        def visitNode(self,node):
            thisIsTheNode = 1
            for name,value in predicates.items():
                if not self._nodeMatchesPredicate(node,name,value):
                    thisIsTheNode = 0
            if thisIsTheNode:
                self.node = node
            else:
                for child in node.getChildNodes():
                    self.visit(child)

    f = FindNode(nodename)
    compiler.walk(ast,f)
    return f.node
