#!/usr/bin/env python 
import sys
sys.path.append("../..")

from bike.refactor.renameMethod import renameMethod
from bike.parsing.load import load
from bike.parsing.output import save, tokensToString, astToString

class StandardIO:
    def write(self,output):
        import sys
        sys.stdout.write(output)
        #print output,
        sys.stdout.flush()

    def read(self):
        return raw_input()


class cli:
    def __init__(self,io=None,showProgress=0):
        self.io = io
        self.showProgress = showProgress
        self.currentAction = None

    def askUserIfShouldRenameMethod(self,node,contextnode):
        if self.io is not None:
            nodestr = tokensToString(node.getNodeTokens())
            contextstr = tokensToString(contextnode.getNodeTokens())            
            self._write("In scope "+contextnode.fqn+":\n")
            self._write("Could not deduce the instance type used in the expression '"+nodestr+"'\n")
            self._write("Context:\n\n")
            self._write(contextstr)
            self._write("\n")
            self._write("Should I rename this?")
            if self.io.read() == "y":
                return 1
        return 0

    def _write(self,text):
        if self.io is not None:
            self.io.write(text)

    def doRenameMethod(self,path,classname,method,newname):
        progresscallback = None
        if self.showProgress:
            progresscallback = self.printDot
        ast = load(path,progresscallback)
        numMethods, numRefs = renameMethod(ast,classname,method,newname,promptCallback=self.askUserIfShouldRenameMethod,progressCallback=progresscallback)
        save(ast,progresscallback)
        return numMethods, numRefs

    def printDot(self,action,filename):
        if self.currentAction != action:
            if self.currentAction == None:
                self._write(action)
            else: self._write("\n"+action)
            self.currentAction = action
        self._write(".")
    
    def doInteractiveRenameMethod(self,path,classname,method,newname):
        numMethods, numRefs = self.doRenameMethod(path,classname,method,newname)
        self._write("\n")
        if numMethods == 1:
            self._write("1 method was renamed\n")
        else:
            self._write(str(numMethods)+" methods were renamed\n")
        
        if numRefs == 1:
            self._write("1 method reference was renamed\n")
        else:
            self._write(str(numRefs)+" method references were renamed\n")

        if numRefs ==0 and numMethods == 0:
            self._write("Hmmm... nothing was renamed.\n")
            self._write("Did you use a fully qualified classname (including the root module/package)?\n")



if __name__ == "__main__":
    if len(sys.argv) != 5:
        print "Bicycle Repair Man CLI interface."
        print "(Currently does a partial RenameMethod)"
        print "Usage:",sys.argv[0],"file/directory classname methodname newmethodname"
        print "        e.g.  ",sys.argv[0],"foo.py foo.MyClass myMethod newName"
        sys.exit(0)
        
    path,classname,method,newname = sys.argv[1:]
    cli(StandardIO(),1).doInteractiveRenameMethod(path,classname,method,newname)
