#!/usr/bin/env python
import sys
if not "../.."in sys.path: sys.path.append("../..")

from bike.parsing import load
from bike.refactor import renameFunction, renameClass, renameMethod
from bike.parsing.output import save
from bike.parsing.utils import fqn_rcar, fqn_rcdr
from bike.parsing.addtypeinfo import addtypeinfo
import compiler
import compiler.ast
import os


class StandardIO:
    def write(self, output):
        import sys
        sys.stdout.write(output+"\n")
        #print output,
        sys.stdout.flush()

    def read(self):
        return raw_input()

class MuteIO(StandardIO):
    def write(self, output):
        pass

class Server:
    def __init__(self, io = MuteIO()):
        self.io = io

    def inputLoop(self):
        while 1:
            input = raw_input()
            self.io.write("Input was :"+input)
            self.processCommand(input)

    def processCommand(self, cmd):
        command = cmd.split(" ")
        task = command[0]
        args = command[1:]
        if task == "load":
            self.load(args[0])

        if task == "rename":
            savedfiles = self.rename(args[0], int(args[1]), 
                                     int(args[2]), args[3])
            for file in savedfiles:
                self.io.write("BRMUpdated: "+file)

        if task == "gettype":
            node = self._locateNode(args[0], int(args[1]), 
                                    int(args[2]))
            if node:
                if isinstance(node, compiler.ast.Function):
                    self.io.write("BRMResult: Function")
                if isinstance(node, compiler.ast.Class):
                    self.io.write("BRMResult: Class")
        if task == "reload":
            self.reloadSource(args[0])

    def load(self, path):
        path = os.path.expanduser(path)
        path = os.path.abspath(path)
        self.ast = load.load(path, ProgressWriter(self.io).printFilename)
        return "done"

    def rename(self, filename, x, y, newname):
        node = self._locateNode(filename, x, y)
        if node:
            if isinstance(node, compiler.ast.Function):
                if isinstance(node.parentScope, compiler.ast.Class):
                    self.io.write("BRMDebug: Is a method")
                    renameMethod.renameMethod(self.ast, fqn_rcdr(node.fqn), 
                                              fqn_rcar(node.fqn), newname, 
                                              promptCallback = self.promptUser)
                else:
                    renameFunction.renameFunction(self.ast, node.fqn, newname)
            if isinstance(node, compiler.ast.Class):
                renameClass.renameClass(self.ast, node.fqn, newname)
        savedFiles = save(self.ast)
        self.io.write("done")
        return savedFiles

    def promptUser(self, node, contextnode):
        self.io.write(self._getPromptStr(node))
        response = raw_input()
        self.io.write("BRMDebug response was "+response)
        if response == "y":
            return 1
        return 0

    def _getPromptStr(self,node):
        scoords = node.getStartCoords()
        ecoords = node.getEndCoords()
        return "BRMPromptForRename: "+node.sourcenode.filename+ " "+ str(scoords.x)+" "+str(scoords.y)+" "+str(ecoords.x)+" "+str(ecoords.y)

    def reloadSource(self, filename):
        top = self.ast
        io = self.io
        class SourceVisitor:
            def visitSource(self, node):
                if node.filename == filename:
                    f = open(filename)
                    src = f.read()
                    f.close()
                    node.reload(src)

        compiler.walk(self.ast, SourceVisitor())



    def _locateNode(self, filename, x, y):
        class SourceVisitor:
            def visitSource(self, node):
                if node.filename == filename:
                    node.tokenize()
                    self.visit(node.module)

            def visitFunction(self, node):
                ycoord = node.parsetree[0][2]
                if ycoord == y:
                    tokens = node.getNodeTokens()
                    xstart = tokens[1].start[1]
                    xend = tokens[1].end[1]
                    if x >= xstart and x < xend:
                        self.result = node

            def visitClass(self, node):
                ycoord = node.parsetree[0][2]
                if ycoord == y:
                    tokens = node.getNodeTokens()
                    xstart = tokens[1].start[1]
                    xend = tokens[1].end[1]
                    if x >= xstart and x < xend:
                        self.result = node
                        return
                for child in node.getChildNodes():
                    self.visit(child)

        try:
            return compiler.walk(self.ast, SourceVisitor()).result
        except AttributeError: # didn't find it
            return None

class ProgressWriter:
    def __init__(self, io):
        self.io = io

    def _write(self, txt):
        self.io.write(txt)

    def printDot(self, action, filename):
        import sys
        if not hasattr(self, "currentAction"):
            self._write(action)
        elif self.currentAction != action:
            self._write("\n"+action)
        self.currentAction = action
        self._write(".")

    def printFilename(self, action, filename):
        import sys
        self._write("BRMMessage: "+action+" "+filename)



if __name__ == "__main__":
    Server(StandardIO()).inputLoop()
