#!/usr/bin/env python 
import sys
sys.path.append("../..")

import compiler
from Tkinter import *
from bike.ui.TreeWidget import *

from bike.parsing.load import load
from bike.refactor.renameMethod import renameMethod
from bike.refactor.renameClass import renameClass
from bike.refactor.renameFunction import renameFunction
from bike.parsing.output import save, astToString

class Item(TreeItem):
    def __init__(self,name):
        self.name = name
        self.subitems = []
        self.expandable = 0

    def GetIconName(self):
        return "folder"

    def GetText(self):
        return self.name

    def IsExpandable(self):
        return self.expandable

    def addSubItem(self,item):
        self.expandable = 1
        self.subitems.append(item)
        return item

    def GetSubList(self):
        return self.subitems

    def setTreeNode(self,node):
        self.treeNode = node


class ClassItem(Item):
    def __init__(self,node,controller):
        Item.__init__(self,node.name)
        self.controller = controller
        self.tk = controller.gui
        self.node = node

    def GetIconName(self):
        return "tk"

    def OnDoubleClick(self):
        r = RenameClassDialog(self.tk,self.node.name)
        if r.getresult() != "":
            self.controller.renameClass(self.node.fqn,r.getresult())
        self.name = self.node.name
        if hasattr(self,"treeNode"):
            self.treeNode.label['text'] = self.name
            self.treeNode.drawtext()

class MethodItem(Item):
    def __init__(self,node,controller):
        Item.__init__(self,node.name)
        self.controller = controller
        self.tk = controller.gui
        self.node = node
    
    def GetIconName(self):
        return "python"
    
    def OnDoubleClick(self):
        r = RenameMethodDialog(self.tk,self.node.name)
        if r.getresult() != "":
            self.controller.renameMethod(self.node.fqn,r.getresult())
        self.changeLabel(self.node.name)

    def changeLabel(self,newtext):
        self.name = self.node.name
        if hasattr(self,"treeNode"):
            self.treeNode.label['text'] = newtext
            self.treeNode.drawtext()

class FunctionItem(MethodItem):
    def OnDoubleClick(self):
        r = RenameFunctionDialog(self.tk,self.node.name)
        if r.getresult() != "":
            self.controller.renameFunction(self.node.fqn,r.getresult())
        self.changeLabel(self.node.name)


class RenameMethodDialog(Toplevel):
    def __init__(self,parent,name):
        Toplevel.__init__(self,parent)
        self.createWidgets()
        self.result=""
        self.goModal()

    def message(self):
        return "Rename Method To"

    def createWidgets(self):
        topbox =  Frame(self)        
        w = Label(topbox, text=self.message(), justify=LEFT)
        w.grid(row=0, padx=5, sticky=W)
        self.entry = Entry(topbox, name="entry")
        self.entry.grid(row=1, padx=5, sticky=W+E)
        topbox.pack()

        box = Frame(self)
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=5)
        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)
        box.pack()

    def goModal(self):
        if TestCallbacks.renameMethodCallback:
            TestCallbacks.renameMethodCallback(self)
        else:
            self.wait_window()
        
    def getresult(self):
        return self.result

    def ok(self, event=None):
        self.result = self.entry.get()
        self.destroy()

    def cancel(self, event=None):
        self.destroy()

class RenameClassDialog(RenameMethodDialog):

    def message(self):
        return "Rename Class To"

    def goModal(self):
        if TestCallbacks.renameClassCallback:
            TestCallbacks.renameClassCallback(self)
        else:
            self.wait_window()

class RenameFunctionDialog(RenameMethodDialog):
    def message(self):
        return "Rename Function To"

    def goModal(self):
        if TestCallbacks.renameMethodCallback:
            TestCallbacks.renameMethodCallback(self)
        else:
            self.wait_window()

class AskUserIfShouldRenameMethodReferenceDialog(Toplevel):
    def __init__(self,parent,scope,node,context):
        Toplevel.__init__(self,parent)
        self.createWidgets(scope,node,context)
        self.goModal()

    def goModal(self):
        if TestCallbacks.renameMethodReferenceCallback:
            TestCallbacks.renameMethodReferenceCallback(self)
        else:
            self.wait_window()

    def createWidgets(self,scope,nodestr,context):
        l = Label(self,text="In scope "+scope)
        l.pack()
        l = Label(self,justify='left',text="Could not deduce the instance type\n used in the expression '"+nodestr+"'\n\nContext:")
        l.pack()
        m = Message(self,text=context,width=10000,font=('courier',12))
        m.pack()
        l = Label(self,text="Should I rename this method reference?")
        l.pack()
        box = Frame(self)
        w = Button(box, text="Yes", width=10, command=self.yes, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="No", width=10, command=self.no)
        w.pack(side=LEFT, padx=5, pady=5)
        box.pack()

    def getAnswer(self):
        return self.answer

    def yes(self):
        self.answer = 1
        self.destroy()
    
    def no(self):
        self.answer = 0
        self.destroy()


# these are used so that unittests can check gui logic
class TestCallbacks:
    renameMethodCallback = None
    renameMethodReferenceCallback = None
    renameClassCallback = None

class BikeGUI(Frame):
    def __init__(self,controller,master):
        Frame.__init__(self,master)
        self.master = master
        self.controller = controller
        self.sc = ScrolledCanvas(self, bg="white", highlightthickness=0, takefocus=1)
        self.pack(expand=1, fill="both",side="top")
        buttonbox = Frame(self)
        buttonbox.grid(row=2,column=0)
        Button(buttonbox,text="Save",command=self.save).pack(side="left")
        Button(buttonbox,text="Quit",command=self.quit).pack(side="right")
        
        item = Item("Root")
        self.root = item
        self.tree = TreeNode(self.sc.canvas, None, item)

    def quit(self):
        self.master.destroy()

    def save(self):
        self.controller.saveAST()

    def askUserIfShouldRenameMethodReference(self,node,contextnode):
        scopestr = contextnode.fqn
        nodeline = node.getStartCoords().y
        contextbeginline = contextnode.getStartCoords().y
        nodestr = astToString(node)
        contextstr = addPointerToSource(astToString(contextnode),
                                        (nodeline - contextbeginline) + 1)
        a = AskUserIfShouldRenameMethodReferenceDialog(self,scopestr,nodestr,contextstr)
        return a.getAnswer()


# adds a '-->' to the source code on the given line
def addPointerToSource(src,linenumber):
    array = src.split("\n")
    newstr = ""
    currentlineno = 0
    for line in array:
        currentlineno +=1
        if currentlineno == linenumber:
            newstr +=  "-->"+line+"\n"
        else:
            newstr +=  "   "+line+"\n"
    return newstr

        
class BikeController:
    def __init__(self,rootdir,**args):
        self.tk = Tk()
        self.gui = BikeGUI(self,self.tk)
        if args.has_key('progressCallback'):
            self.ast = load(rootdir,args['progressCallback'])
        else:
            self.ast = load(rootdir)
        self.args = args
        self.populateTree(self.ast)
        
    def populateTree(self,ast):
        root = self.gui.root
        controller = self  # so that sub class can reach it
        class ASTVisitor:
            def visitTop(self,node):
                for c in node.getChildNodes():
                    self.visit(c,root)

            def visitPackage(self,node,item):
                packageitem = Item(node.name)
                item.addSubItem(packageitem)
                for c in node.getChildNodes():
                    self.visit(c,packageitem)

            def visitSource(self,node,parentitem):
                moduleitem = Item(node.modulename)
                parentitem.addSubItem(moduleitem)
                self.visit(node.module,moduleitem)
            
            def visitClass(self,node,parentitem,parentClass=None):
                classitem = ClassItem(node,controller)
                parentitem.addSubItem(classitem)
                for c in node.getChildNodes():
                    self.visit(c,classitem,node)

            def visitFunction(self,node,parentitem,parentClass=None):
                childnodes = node.getChildNodes()
                if parentClass:
                    item = MethodItem(node,controller)
                else:
                    item = FunctionItem(node,controller)
                parentitem.addSubItem(item)
                for c in node.getChildNodes():
                    self.visit(c,item)
                    
        compiler.walk(ast,ASTVisitor())
        self.gui.tree.expand()

    def renameMethod(self,fqn,newname):
        classname = fqn[:fqn.rindex('.')]
        methodname = fqn[fqn.rindex('.')+1:]
        renameMethod(self.ast,classname,methodname,newname,
                     promptCallback = self.gui.askUserIfShouldRenameMethodReference)

    def renameClass(self,fqn,newname):
        renameClass(self.ast,fqn,newname)

    def renameFunction(self,fqn,newname):
        renameFunction(self.ast,fqn,newname)

    def saveAST(self):
        savedFiles = save(self.ast)
        for i in savedFiles:
            if self.args.has_key("progressCallback"):
                self.args['progressCallback']("saving",i)

    def mainloop(self):
        self.gui.mainloop()

class PrintDot:
    def _write(self,txt):
        sys.stdout.write(txt)
        sys.stdout.flush()
            
    def printDot(self,action,filename):
        import sys
        if not hasattr(self,"currentAction"):
            self._write(action)
        elif self.currentAction != action:
            self._write("\n"+action)
        self.currentAction = action
        self._write(".")

    def printFilename(self,action,filename):
        import sys
        print action,filename
        
if __name__ == "__main__":
    if len(sys.argv) != 2:
        print "Bicycle Repair Man GUI interface."
        print "(Currently does RenameMethod and RenameClass)"
        print "Usage:",sys.argv[0],"[python file / package directory]"
        sys.exit(0)
    #b = BikeController(sys.argv[1],progressCallback=PrintDot().printDot)
    b = BikeController(sys.argv[1],progressCallback=PrintDot().printFilename)
    b.mainloop()
    print ""
