#!/usr/bin/env python
from __future__ import nested_scopes
from bikegui import *
import unittest
import os
from bike import testdata
from bike.testutils import *
from bike.refactor.test_renameMethod import RenameMethodTests, RenameMethodReferenceTests_ImportsClass, RenameMethodReferenceTests_doesntImportClass, RenameMethodAfterPromptTests
from bike.refactor.test_renameClass import RenameClassTests, RenameClassTests_importsClass
from bike.refactor.test_renameFunction import RenameFunctionTests, RenameFunctionTests_importsFunction
from bike.parsing.brmtransformer import parse
from bike.parsing.addtokens import addTokens

# get the first element of a fully qualified python path
#(e.g. _car('a.b.c.d') = 'a')
def _car(fqn):
    try:
        return fqn[:fqn.index(".")]
    except ValueError:   # i.e. no dots in fqn
        return fqn

# get the other elements of a fully qualified python path
#(e.g. _cdr('a.b.c.d') = 'b.c.d')
def _cdr(fqn):
    try:
        return fqn[fqn.index(".")+1:]
    except ValueError:   # i.e. no dots in fqn
        return ""




# fqn = fully qualified name
def _findGUITreeNode(node,fqn):
    if node.name == _car(fqn) and _cdr(fqn) == "":
        return node
    if node.name == _car(fqn):
        for child in node.GetSubList():
            node = _findGUITreeNode(child,_cdr(fqn))
            if node != None:
                return node


class Test_thisTestModule(unittest.TestCase):
    def test_carReturnsTheFirstElementOfTheFqn(self):
        fqn = "apple.pear.foo"
        assert _car(fqn) == "apple"
        
    def test_carReturnsElementInOneElementFqn(self):
        fqn = "apple"
        assert _car(fqn) == "apple"

    def test_cdrReturnsTheAllElementsOfTheFqnExceptFirst(self):
        fqn = "apple.pear.foo"
        assert _cdr(fqn) == "pear.foo"
        
    def test_cdrReturnsEmptyStringForOneElementFqn(self):
        fqn = "apple"
        assert _cdr(fqn) == ""


    


        
class TestBikeGUI(unittest.TestCase):

    def _typeNewMethodNameAndHitCancel(self,dialog):
        dialog.cancel()
        dialog.result = "foobah"

    def _typeNewMethodNameAndHitOk(self,dialog):
        dialog.ok()
        dialog.result = self.newname

    def _clickYes(self,dialog):
        dialog.yes()
        self._clickYesCalled = 1

    def _clickNo(self,dialog):
        dialog.no()

    def runTarget(self,src,klass,method,newname):
        createPackageStructure("pass",src)
        bc = BikeController("a")
        self.newname = newname
        fngui = _findGUITreeNode(bc.gui.root,"Root.a.b.bah."+klass+"."+method)
        TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
        fngui.OnDoubleClick() # click the tree element
        bc.gui.save() # click the save button
        bc.tk.destroy()
        ast = load("a")    
        removePackageStructure()
        return ast

    def test_hittingCancelDoesntRenameMethod(self):
        src = testdata.class_and_method
        createPackageStructure("pass",src)
        g = BikeController("a")
        fngui = _findGUITreeNode(g.gui.root,"Root.a.b.bah.a.foo")
        TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitCancel
        fngui.OnDoubleClick() # click the tree element
        g.tk.destroy()
        srcafter = readFile("a/b/bah.py")
        removePackageStructure()
        assert src == srcafter
    
    def test_RenamesMethod(self):
        src = testdata.class_and_method
        ast = self.runTarget(src,"a","foo","newName")
        fnnode = getASTNode(ast,"Function")
        assert fnnode.name == "newName"


    def test_promptsUserIfCantDeduceTypeOfMethod(self):
        src = testdata.ObjectRefWithNoDeducableType
        self._clickYesCalled = 0
        TestCallbacks.renameMethodReferenceCallback = self._clickYes
        ast = self.runTarget(src,"TheClass","theMethod","newName")
        assert self._clickYesCalled == 1
        getattrnode = getASTNode(ast,"Getattr",1)
        assert getattrnode.attrname == "newName"

    def test_promptsUserButDoesntRenameReferenceIfUserSaysNo(self):
        src = testdata.ObjectRefWithNoDeducableType
        self._clickYesCalled = 0
        TestCallbacks.renameMethodReferenceCallback = self._clickNo
        ast = self.runTarget(src,"TheClass","theMethod","newName")
        getattrnode = getASTNode(ast,"Getattr",1)
        assert getattrnode.attrname == "theMethod"

    def test_inputSrcEqualsOutputSrcIfMethodNotRenamed(self):
        src = testdata.source_with_all_ast_nodes[1:]
        createPackageStructure(src,"pass")
        bc = BikeController("a")
        bc.gui.save() # click the save button
        bc.tk.destroy()
        newsrc = readFile(os.path.join("a","foo.py"))
        removePackageStructure()
        assert src == newsrc


class TestRenameMethodThroughGUI(RenameMethodTests,unittest.TestCase):
    def _typeNewMethodNameAndHitCancel(self,dialog):
        dialog.cancel()
        dialog.result = "foobah"

    def _typeNewMethodNameAndHitOk(self,dialog):
        dialog.ok()
        dialog.result = self.newname

    def _clickYes(self,dialog):
        dialog.yes()
        self._clickYesCalled = 1

    def _clickNo(self,dialog):
        dialog.no()

    def runTarget(self,src,klass,method,newname):
        try:
            createPackageStructure('pass',src)
            bc = BikeController("a")
            self.newname = newname
            fngui = _findGUITreeNode(bc.gui.root,"Root.a.b.bah."+klass+"."+method)
            TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
            TestCallbacks.renameMethodReferenceCallback = None
            fngui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            ast = load("a")
        finally:
            try:
                removePackageStructure()
                bc.tk.destroy()
            except: pass
        return ast

class RenameStuffThroughGUIHelper:
    def _typeNewMethodNameAndHitCancel(self,dialog):
        dialog.cancel()
        dialog.result = "foobah"

    def _typeNewMethodNameAndHitOk(self,dialog):
        dialog.ok()
        dialog.result = self.newname

    def _clickYes(self,dialog):
        dialog.yes()
        self._clickYesCalled = 1

    def _clickNo(self,dialog):
        dialog.no()
    

class TestRenameMethodReferenceThroughGUI_NoImports(RenameStuffThroughGUIHelper,RenameMethodReferenceTests_doesntImportClass,unittest.TestCase):

    def runTarget(self,src,klass,method,newname):
        try:
            writeTmpTestFile(src)
            bc = BikeController(tmpfile)
            self.newname = newname
            fngui = _findGUITreeNode(bc.gui.root,"Root."+tmpmodule+"."+klass+"."+method)
            TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
            TestCallbacks.renameMethodReferenceCallback = self._clickNo
            fngui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(tmpfile)
            ast = parse(newsrc)
            addTokens(ast,newsrc)
        finally:
            bc.tk.destroy()
            deleteTmpTestFile()
        return ast

class TestRenameMethodReferenceThroughGUI_WithImports(RenameStuffThroughGUIHelper,RenameMethodReferenceTests_ImportsClass,unittest.TestCase):
    def runTarget(self,src,klass,method,newname):
        try:
            createPackageStructure(src,testdata.TheClass)
            bc = BikeController("a")
            self.newname = newname
            fngui = _findGUITreeNode(bc.gui.root,"Root."+klass+"."+method)
            TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
            TestCallbacks.renameMethodReferenceCallback = self._clickNo
            fngui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(os.path.join("a","foo.py"))
            ast = Source("","",newsrc)
        finally:
            bc.tk.destroy()
            removePackageStructure()
        return ast


class TestRenameClassThroughGUI_NoImports(RenameStuffThroughGUIHelper,RenameClassTests,unittest.TestCase):

    def runTarget(self,src,klass,newname):
        try:
            writeTmpTestFile(src)
            bc = BikeController(tmpfile)
            self.newname = newname
            classgui = _findGUITreeNode(bc.gui.root,"Root."+tmpmodule+"."+klass)
            TestCallbacks.renameClassCallback = self._typeNewMethodNameAndHitOk
            classgui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(tmpfile)
            ast = parse(newsrc)
            addTokens(ast,newsrc)
        finally:
            bc.tk.destroy()
            deleteTmpTestFile()
        return ast

class TestRenameClassThroughGUI_ImportsClass(RenameStuffThroughGUIHelper,RenameClassTests_importsClass,unittest.TestCase):
    def runTarget(self,src,klass,newname):
        try:
            createPackageStructure(src,testdata.TheClass)
            bc = BikeController("a")
            self.newname = newname
            fngui = _findGUITreeNode(bc.gui.root,"Root."+klass)
            TestCallbacks.renameClassCallback = self._typeNewMethodNameAndHitOk
            fngui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(os.path.join("a","foo.py"))
            ast = Source("","",newsrc)
        finally:
            bc.tk.destroy()
            removePackageStructure()
        return ast



class TestRenameFunctionThroughGUI_NoImports(RenameStuffThroughGUIHelper,RenameFunctionTests,unittest.TestCase):

    def runTarget(self,src,fqn,newname):
        try:
            writeTmpTestFile(src)
            bc = BikeController(tmpfile)
            self.newname = newname
            classgui = _findGUITreeNode(bc.gui.root,"Root."+tmpmodule+"."+fqn)
            TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
            classgui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(tmpfile)
            ast = parse(newsrc)
            addTokens(ast,newsrc)
        finally:
            bc.tk.destroy()
            deleteTmpTestFile()
        return ast


class TestRenameFunctionThroughGUI_ImportsFunction(RenameStuffThroughGUIHelper,RenameFunctionTests_importsFunction,unittest.TestCase):
    def runTarget(self,src,fqn,newname):
        try:
            createPackageStructure(src,testdata.Function)
            bc = BikeController("a")
            self.newname = newname
            fngui = _findGUITreeNode(bc.gui.root,"Root."+fqn)
            TestCallbacks.renameMethodCallback = self._typeNewMethodNameAndHitOk
            fngui.OnDoubleClick() # click the tree element
            bc.gui.save() # click the save button
            newsrc = readFile(os.path.join("a","foo.py"))
            ast = Source("","",newsrc)
        finally:
            bc.tk.destroy()
            removePackageStructure()
        return ast


class TestAddPointerToSource(unittest.TestCase):
    def test_AddsPointerToSource(self):
        src = testdata.class_and_method[1:]
        newsrc = addPointerToSource(src,2)
        assert newsrc == testdata_addpointerToSource_output[1:]

testdata_addpointerToSource_output="""
   class a:
-->    def foo():
           pass
   
"""

if __name__ == "__main__":
    unittest.main()
