#!/usr/bin/env python
import compiler
import sys
sys.path.append("..") 

from bike.parsing.load import load_readonly
from bike.parsing.extended_ast import Top
from pyxmi.xmilib import *


def genxmi(modelname,paths,outputfilename,progressPrinter=None):
    top = Top([],[],[])
    for path in paths:
        ast = load_readonly(path,progressPrinter,top)
    if progressPrinter:
        print "CreatingXML..",
    doc = compiler.walk(ast,XMIVisitor(modelname)).getDoc()
    compiler.walk(ast,PassTwo_AddGeneralizations())
    if progressPrinter:
        print "Saving.."
    f = open(outputfilename,"w+")
    f.write(doc.toxml())
    f.close()

def silent(self,action,filename):
    pass


class PrintDot:
    def _write(self,txt):
        sys.stdout.write(txt)
        sys.stdout.flush()
            
    def printDot(self,action,filename):
        import sys
        if not hasattr(self,"currentAction"):
            self._write(action)
        elif self.currentAction != action:
            self._write("\n"+action)
        self.currentAction = action
        self._write(".")
        

    
class XMIVisitor:
    def __init__(self,name):
        self.root = XMIRoot(name)
        
    def visitPackage(self,package,parentElem=None):
        if parentElem is None:
            parentElem = self.root
        elem = parentElem.addPackage(package.name)
        for child in package.getChildNodes():
            self.visit(child,elem)

    def visitModule(self,module,parentElem=None):
        if parentElem is None:
            parentElem = self.root
        elem = parentElem.addPackage(module.name)
        module.xmielem = elem
        for child in module.getChildNodes():
            self.visit(child,elem,module)

    def visitClass(self,klass,parentElem=None,parent=None):
        if parentElem is None:
            parentElem = self.root
        if isinstance(parentElem,XMIPackage):
            elem = parentElem.addClass(klass.name)
            klass.xmielem = elem
            for child in klass.getChildNodes():
                self.visit(child,elem,klass)

    def visitFunction(self,node,parentElem=None,parent=None):
        if isinstance(parent,compiler.ast.Class):
            elem = parentElem.addOperation(node.name)
            for argname in node.argnames:
                elem.addParameter(argname)

    def getDoc(self):
        return self.root.doc



class PassTwo_AddGeneralizations:

    def visitModule(self,module,parentElem=None):
        for child in module.getChildNodes():
            self.visit(child,module)

    def visitClass(self,klass,parent):
        for baseclassname in klass.bases:
            baseclass = klass.typeinfo.getTypeOfExpr(baseclassname)    
            if baseclass:
                parent.xmielem.addGeneralisation(baseclass.xmielem,klass.xmielem)

if __name__ == "__main__":
    import sys
    if len(sys.argv) < 4:
        print "\tUsage:",sys.argv[0],"path(s) xmifile modelname"
        sys.exit(0)
    infiles = sys.argv[1:-2]
    outfile,name = sys.argv[-2:]
    genxmi(name,infiles,outfile,PrintDot().printDot)
