#!/usr/bin/env python
import sys
sys.path.append("..") 
import unittest
from py2xmi import genxmi
from bike.testutils import *
from bike import testdata

class TestGeneralizations(unittest.TestCase):
    def testfoo(self):
        createPackageStructure(testdata.ClassWithNestedInheritanceSpec,
                               testdata.TheClass)
        genxmi("mymodel","a",tmpfile)
        deleteTmpTestFile()
        
        removePackageStructure()


class TestGenXMI(unittest.TestCase):
    def testWorksWithPackageAndClass(self):
        createPackageStructure(input_classWithOperations,"")
        genxmi("mymodel","a",tmpfile)
        xml = readTmpTestFile()
        deleteTmpTestFile()
        removePackageStructure()
        assert xml == output_classWithOperations[1:-1]

    def testHandlesDerivedClasses(self):
        createPackageStructure(input_inheritence,"")
        genxmi("mymodel","a",tmpfile)
        xml = readTmpTestFile()
        deleteTmpTestFile()
        removePackageStructure()
        assert xml == output_inheritence[1:-1]

    def testDoesntBarfOnWierdSyntaxFromASTModule(self):
        createPackageStructure(wierd_syntax_from_ast,"")
        genxmi("mymodel","a",tmpfile)
        deleteTmpTestFile()
        removePackageStructure()

input_classWithOperations="""
class MyClass:
    def myOperation(self,hello):
        pass
    def myOperation2(self,hello):
        pass
"""

output_classWithOperations="""
<?xml version="1.0" ?>
<XMI xmi.version="1.0"><XMI.header><XMI.documentation><XMI.exporter>PyXMI</XMI.exporter><XMI.exporterVersion>0.2</XMI.exporterVersion></XMI.documentation><XMI.metamodel xmi.name="UML" xmi.version="1.3"/></XMI.header><XMI.content><Model_Management.Model xmi.id="xmi.1"><Foundation.Core.ModelElement.name>mymodel</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.2"><Foundation.Core.ModelElement.name>a</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.1"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.3"><Foundation.Core.ModelElement.name>b</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.2"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.4"><Foundation.Core.ModelElement.name>bah</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.3"/></Foundation.Core.ModelElement.namespace></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Package><Model_Management.Package xmi.id="xmi.5"><Foundation.Core.ModelElement.name>foo</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.2"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Foundation.Core.Class xmi.id="xmi.6"><Foundation.Core.ModelElement.name>MyClass</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.Class.isActive xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.5"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Classifier.feature><Foundation.Core.Operation xmi.id="xmi.7"><Foundation.Core.ModelElement.name>myOperation</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.visibility xmi.value="public"/><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.BehavioralFeature.isQuery xmi.value="false"/><Foundation.Core.Operation.isRoot xmi.value="false"/><Foundation.Core.Operation.isLeaf xmi.value="false"/><Foundation.Core.Operation.isAbstract xmi.value="false"/><Foundation.Core.Feature.owner><Foundation.Core.Classifier xmi.idref="xmi.6"/></Foundation.Core.Feature.owner><Foundation.Core.BehavioralFeature.parameter><Foundation.Core.Parameter xmi.id="xmi.8"><Foundation.Core.ModelElement.name>return</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="return"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.7"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter><Foundation.Core.Parameter xmi.id="xmi.9"><Foundation.Core.ModelElement.name>self</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="inout"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.7"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter><Foundation.Core.Parameter xmi.id="xmi.10"><Foundation.Core.ModelElement.name>hello</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="inout"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.7"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter></Foundation.Core.BehavioralFeature.parameter></Foundation.Core.Operation><Foundation.Core.Operation xmi.id="xmi.11"><Foundation.Core.ModelElement.name>myOperation2</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.visibility xmi.value="public"/><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.BehavioralFeature.isQuery xmi.value="false"/><Foundation.Core.Operation.isRoot xmi.value="false"/><Foundation.Core.Operation.isLeaf xmi.value="false"/><Foundation.Core.Operation.isAbstract xmi.value="false"/><Foundation.Core.Feature.owner><Foundation.Core.Classifier xmi.idref="xmi.6"/></Foundation.Core.Feature.owner><Foundation.Core.BehavioralFeature.parameter><Foundation.Core.Parameter xmi.id="xmi.12"><Foundation.Core.ModelElement.name>return</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="return"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.11"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter><Foundation.Core.Parameter xmi.id="xmi.13"><Foundation.Core.ModelElement.name>self</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="inout"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.11"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter><Foundation.Core.Parameter xmi.id="xmi.14"><Foundation.Core.ModelElement.name>hello</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.Parameter.kind xmi.value="inout"/><Foundation.Core.Parameter.behavioralFeature><Foundation.Core.BehavioralFeature xmi.idref="xmi.11"/></Foundation.Core.Parameter.behavioralFeature></Foundation.Core.Parameter></Foundation.Core.BehavioralFeature.parameter></Foundation.Core.Operation></Foundation.Core.Classifier.feature></Foundation.Core.Class></Foundation.Core.Namespace.ownedElement></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Model></XMI.content></XMI>
"""

input_inheritence ="""
class base:
    pass

class derived(base):
    pass

"""

output_inheritence = """
<?xml version="1.0" ?>
<XMI xmi.version="1.0"><XMI.header><XMI.documentation><XMI.exporter>PyXMI</XMI.exporter><XMI.exporterVersion>0.2</XMI.exporterVersion></XMI.documentation><XMI.metamodel xmi.name="UML" xmi.version="1.3"/></XMI.header><XMI.content><Model_Management.Model xmi.id="xmi.1"><Foundation.Core.ModelElement.name>mymodel</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.2"><Foundation.Core.ModelElement.name>a</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.1"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.3"><Foundation.Core.ModelElement.name>b</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.2"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Model_Management.Package xmi.id="xmi.4"><Foundation.Core.ModelElement.name>bah</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.3"/></Foundation.Core.ModelElement.namespace></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Package><Model_Management.Package xmi.id="xmi.5"><Foundation.Core.ModelElement.name>foo</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.2"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace.ownedElement><Foundation.Core.Class xmi.id="xmi.6"><Foundation.Core.ModelElement.name>base</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.Class.isActive xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.5"/></Foundation.Core.ModelElement.namespace><Foundation.Core.GeneralizableElement.specialization><Foundation.Core.Generalization xmi.idref="xmi.8"/></Foundation.Core.GeneralizableElement.specialization></Foundation.Core.Class><Foundation.Core.Class xmi.id="xmi.7"><Foundation.Core.ModelElement.name>derived</Foundation.Core.ModelElement.name><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.GeneralizableElement.isRoot xmi.value="false"/><Foundation.Core.GeneralizableElement.isLeaf xmi.value="false"/><Foundation.Core.GeneralizableElement.isAbstract xmi.value="false"/><Foundation.Core.Class.isActive xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.5"/></Foundation.Core.ModelElement.namespace><Foundation.Core.GeneralizableElement.generalization><Foundation.Core.Generalization xmi.idref="xmi.8"/></Foundation.Core.GeneralizableElement.generalization></Foundation.Core.Class><Foundation.Core.Generalization xmi.id="xmi.8"><Foundation.Core.ModelElement.isSpecification xmi.value="false"/><Foundation.Core.ModelElement.namespace><Foundation.Core.Namespace xmi.idref="xmi.5"/></Foundation.Core.ModelElement.namespace><Foundation.Core.Generalization.child><Foundation.Core.GeneralizableElement xmi.idref="xmi.7"/></Foundation.Core.Generalization.child><Foundation.Core.Generalization.parent><Foundation.Core.GeneralizableElement xmi.idref="xmi.6"/></Foundation.Core.Generalization.parent></Foundation.Core.Generalization></Foundation.Core.Namespace.ownedElement></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Package></Foundation.Core.Namespace.ownedElement></Model_Management.Model></XMI.content></XMI>
"""


wierd_syntax_from_ast="""
class Mul(Node):
    nodes["mul"] = "Mul"
    def __init__(self, (left, right)):
        self.left = left
        self.right = right
"""


if __name__ == "__main__":
    unittest.main()
