from xml.dom.minidom import Document
version = 0.2

# metaclass which adds the __super member to instance
class autosuper(type):
    def __init__(cls, name, bases, dict):
        super(autosuper, cls).__init__(name, bases, dict)
        setattr(cls, "_%s__super" % name, super(cls))

class XMINode:
    __metaclass__ = autosuper

    def __init__(self,doc):
        self.doc = doc
        self.id = getid()

    def _addXMLElem(self,parentelem,name,attrlist=[]):
        e = self.doc.createElement(name)
        for name,val in attrlist:
            e.setAttribute(name,val)
        parentelem.appendChild(e)
        return e

    def _addXMLTextElem(self,parentelem,name,text,attrlist=[]):
        e = self._addXMLElem(parentelem,name,attrlist)
        e.appendChild(self.doc.createTextNode(str(text)))

    def _addBooleanElements(self,elemtype):
        self._addXMLElem(self.elem,"Foundation.Core."+elemtype+".isRoot",[['xmi.value',"false"]])
        self._addXMLElem(self.elem,"Foundation.Core."+elemtype+".isLeaf",[['xmi.value',"false"]])
        self._addXMLElem(self.elem,"Foundation.Core."+elemtype+".isAbstract",[['xmi.value',"false"]])


class XMIChildNode(XMINode):
    def __init__(self,parent):
        self.__super.__init__(parent.doc)
        self.parent = parent

    def _addNamespaceElement(self):
        ns = self._addXMLElem(self.elem,"Foundation.Core.ModelElement.namespace")
        self._addXMLElem(ns,"Foundation.Core.Namespace",[["xmi.idref",self.parent.id]])


class XMIGeneralizableElement:
    __metaclass__ = autosuper
    def _addBooleanElements(self):
        self._addXMLElem(self.elem,"Foundation.Core.ModelElement.isSpecification",[['xmi.value',"false"]])
        self.__super._addBooleanElements("GeneralizableElement")

    


class XMIOwnsElements:

    def getOwnedElem(self):
        if not hasattr(self,"ownedelem"):
            self.ownedelem = self._addXMLElem(self.elem,"Foundation.Core.Namespace.ownedElement")
        return self.ownedelem


class XMIRoot(XMIGeneralizableElement,XMINode,XMIOwnsElements):
    def __init__(self,name):
        resetid()
        self.__super.__init__(Document())
        root = self._addXMLElem(self.doc,"XMI",[["xmi.version","1.0"]])
        header = self._addXMLElem(root,"XMI.header")
        documentation = self._addXMLElem(header,"XMI.documentation")
        self._addXMLTextElem(documentation,"XMI.exporter","PyXMI")
        self._addXMLTextElem(documentation,"XMI.exporterVersion",str(version))
        e = self._addXMLElem(header,"XMI.metamodel",[["xmi.name","UML"],["xmi.version","1.3"]])
        content = self._addXMLElem(root,"XMI.content")
        self.elem = self._addXMLElem(content,"Model_Management.Model",[["xmi.id",self.id]])
        self._addXMLTextElem(self.elem,"Foundation.Core.ModelElement.name",name)
        self._addBooleanElements()

    def addPackage(self,name):
        return XMIPackage(self,name)
        

class XMIPackage(XMIGeneralizableElement,XMIChildNode,XMIOwnsElements):
    def __init__(self,parent,name):
        self.__super.__init__(parent)
        self.elem = self._addXMLElem(parent.getOwnedElem(),"Model_Management.Package",[["xmi.id",self.id]])
        e = self._addXMLTextElem(self.elem,"Foundation.Core.ModelElement.name",name)
        self._addBooleanElements()
        self._addNamespaceElement()

    def addPackage(self,name):
        return XMIPackage(self,name)
    
    def addClass(self,name):
        return XMIClass(self,name)

    def addGeneralisation(self,baseclasselem,subclasselem):
        return XMIGeneralization(self,baseclasselem,subclasselem)

class XMIClass(XMIGeneralizableElement,XMIChildNode):
    def __init__(self,parent,name):
        self.__super.__init__(parent)
        self.elem = self._addXMLElem(parent.getOwnedElem(),"Foundation.Core.Class",[["xmi.id",self.id]])
        e = self._addXMLTextElem(self.elem,"Foundation.Core.ModelElement.name",name)
        self._addBooleanElements()
        self._addNamespaceElement()

    #def addClass(self,name):
    #    return XMIClass(self,name)

    def getFeatureElem(self):
        if not hasattr(self,"featureelem"):
            self.featureelem = self._addXMLElem(self.elem,"Foundation.Core.Classifier.feature")
        return self.featureelem
        
    def addOperation(self,name):
        return XMIOperation(self,name)

    def addSpecializationElement(self,subclasselem):
        sp = self._addXMLElem(self.elem,"Foundation.Core.GeneralizableElement.specialization")
        self._addXMLElem(sp,"Foundation.Core.Generalization",[["xmi.idref",subclasselem.id]])
        
    def addGeneralizationElement(self,baseclasselem):
        gen = self._addXMLElem(self.elem,"Foundation.Core.GeneralizableElement.generalization")
        self._addXMLElem(gen,"Foundation.Core.Generalization",[["xmi.idref",baseclasselem.id]])

    def _addBooleanElements(self):
        self.__super._addBooleanElements()
        self._addXMLElem(self.elem,"Foundation.Core.Class.isActive",[['xmi.value',"false"]])



class XMIOperation(XMIChildNode):
    def __init__(self,parent,name):
        self.__super.__init__(parent)
        self.elem = self._addXMLElem(parent.getFeatureElem(),"Foundation.Core.Operation",[["xmi.id",self.id]])
        self._addXMLTextElem(self.elem,"Foundation.Core.ModelElement.name",name)
        self._addXMLElem(self.elem,"Foundation.Core.ModelElement.visibility",[["xmi.value","public"]])
        self._addBooleanElements()
        self._addFeatureOwnerElement()
        self.addParameter("return","return")

    def _addBooleanElements(self):
        self._addXMLElem(self.elem,"Foundation.Core.ModelElement.isSpecification",[['xmi.value',"false"]])
        self._addXMLElem(self.elem,"Foundation.Core.BehavioralFeature.isQuery",[['xmi.value',"false"]])
        self.__super._addBooleanElements("Operation")
    
    def addParameter(self,name,kind="inout"):
        return XMIParameter(self,name,kind)

    def _addFeatureOwnerElement(self):
        ns = self._addXMLElem(self.elem,"Foundation.Core.Feature.owner")
        self._addXMLElem(ns,"Foundation.Core.Classifier",[["xmi.idref",self.parent.id]])

    def getBehavioralFeatureElem(self):
        if not hasattr(self,"featureelem"):
            self.featureelem = self._addXMLElem(self.elem,"Foundation.Core.BehavioralFeature.parameter")
        return self.featureelem


class XMIParameter(XMIChildNode):
    def __init__(self,parent,name,kind):
        self.__super.__init__(parent)
        self.elem = self._addXMLElem(parent.getBehavioralFeatureElem(),"Foundation.Core.Parameter",[["xmi.id",self.id]])
        self._addXMLTextElem(self.elem,"Foundation.Core.ModelElement.name",name)
        self._addBooleanElements()
        self._addXMLElem(self.elem,"Foundation.Core.Parameter.kind",[["xmi.value",kind]])
        self._addBehavioralFeatureOwnerElement()


    def _addBooleanElements(self):
        self._addXMLElem(self.elem,"Foundation.Core.ModelElement.isSpecification",[['xmi.value',"false"]])


    def _addBehavioralFeatureOwnerElement(self):
        ns = self._addXMLElem(self.elem,"Foundation.Core.Parameter.behavioralFeature")
        self._addXMLElem(ns,"Foundation.Core.BehavioralFeature",[["xmi.idref",self.parent.id]])

class XMIGeneralization(XMIChildNode):
    def __init__(self,parent,baseclass,subclass):
        self.__super.__init__(parent)
        self.elem = self._addXMLElem(parent.getOwnedElem(),"Foundation.Core.Generalization",[["xmi.id",self.id]])
        self._addXMLElem(self.elem,"Foundation.Core.ModelElement.isSpecification",[["xmi.value","false"]])
        self._addNamespaceElement()
        c = self._addXMLElem(self.elem,"Foundation.Core.Generalization.child")
        self._addXMLElem(c,"Foundation.Core.GeneralizableElement",[["xmi.idref",subclass.id]])
        c = self._addXMLElem(self.elem,"Foundation.Core.Generalization.parent")
        self._addXMLElem(c,"Foundation.Core.GeneralizableElement",[["xmi.idref",baseclass.id]])
        self.addGeneralizationReferencesToClassElements(baseclass,subclass)
        
    def addGeneralizationReferencesToClassElements(self,baseclass,subclass):
        baseclass.addSpecializationElement(self)
        subclass.addGeneralizationElement(self)


id = 0

def getid():
    global id
    id += 1
    return "xmi."+str(id)

def resetid():
    global id
    id = 0

